## Read user attributes

Use the username or an email address to get user data. The 2nd parameter defines the attributes to fetch (`["*"]` is default).

```php
$aUser = $oLdap->getUserInfo("john@example.com", []);
$aUser = $oLdap->getUserInfo("john@example.com", ["memberof", "uid"]);        

// simplify result array:
print_r($oLdap->normalizeSearchentry($aUser));
```

## Verify user and password for login

```php
// set values from $_FORM or $_POST data of your login form here
// The variable $bAuthenticated is true if authentication of the user was successful.
$bAuthenticated=oLdap->verifyPassword($sUser, $sPassword);
```

## Search

When using special chars in search then you can sanitize the search string.

```php
$sCn = 'John Smith (john)';
$sSearchFilter = '(cn='.$oLdap->sanitizeFilter($sCn).')';
$aResults = $oLdap->searchDn("<DN here>", $sSearchFilter, ["*"]);
$oLdap->close();
```

## Debugging

If you want to find connection problems then use debugOn().
This enables the echoing of ldap actions for connect, bind and class internal debug messages.

```php
$oLdap = new imlldap($aConfig);

// enable showing debug output
// This method sets LDAP_OPT_DEBUG_LEVEL to $aConfig['debugLevel'];
$oLdap->debugOn();

// then do something ... the first action will conect and bind
```

## Add a member

This snippet is a function to add eg a user DN as member to an application or other group.

```php
/*
string  $sDn         DN i.e. of an application / group that has member entries
string  $sMemberDN   DN of a user or for smb group: uid of member 
string  $sAttribute  attribute to handle; default: member; for smb group "memberUid"
*/

$oLdap=new imlldap($aConfig['ldap-master']);
if ($oLdap->objectAttributeAndValueExist($sDn, $sAttribute, $sMemberDN)) {
    // SKIP - $sMemberDN is a member in $sDn already
    // ...
} else {
    if ($oLdap->objAddAttr($sDn, array($sAttribute => $sMemberDN))) {
        // OK - member was added
        // ...
    } else {
        // ERROR - adding the member failed
        // ...
    }
}
$oLdap->close();
```
